/****************************************************************************
*
* FILENAME
*   SSTREAM.H
*
* CLASSNAME
*   SerialBuf
*
* PURPOSE
*    A serial streambuf class.
*
* DESCRIPTION
*    This class has all the features of class streambuf.
*
* INHERITANCE
*   public:
*   class streambuf
*
* MEMBER FUNCTIONS
*   public:
*   int SerialBuf::overflow    (int)
*       SerialBuf::SerialBuf   (SerialBuf &)
*       SerialBuf::SerialBuf   (Serial *)
*       SerialBuf::~SerialBuf  ()
*   int SerialBuf::sync        ()
*   int SerialBuf::underflow   ()
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PREPROCESSOR EXCLUSION START
*
*****************************************************************************/

#ifndef __cplusplus
#error Must use C++ for the type SerialStream.
#endif

#ifndef __LARGE__
#error Must use large memory model for the type SerialStream.
#endif

#ifndef __SSTREAM_H
#define __SSTREAM_H

/****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include <iostream.h>  // I/O stream classes
#include "serial.h"    // class Serial

/****************************************************************************
*
* CLASS DEFINITION
*
*****************************************************************************/

class SerialBuf
: public streambuf
{
  private:
  Serial *serialObject;  // Pointer to the class Serial instance
  char buf[2];           // Internal buffer (required)

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialBuf::overflow
*
* PURPOSE
*   Called whenever the internal buffer overflows.
*
* DESCRIPTION
*   Since there is no internal buffer, the buffer overflows on every output
*   operation. The byte that caused the overflow is put in the queue for the
*   serial transmitter.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialBuf
*   class streambuf
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

int                   // RETURN: EOF if operation failed (never!)

SerialBuf::overflow
(
  int c               // INPUT: Byte that caused the overflow
);

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialBuf::SerialBuf
*
* PURPOSE
*   Constructor for class SerialBuf.
*
* DESCRIPTION
*   A pointer to a class Serial object to be used is required.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialBuf
*   class streambuf
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

SerialBuf::SerialBuf
(
  Serial *serialPort  // INPUT: Pointer to class Serial object
);

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialBuf::~SerialBuf
*
* PURPOSE
*   Destructor for class SerialBuf.
*
* DESCRIPTION
*   The destructor does nothing at the present.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialBuf
*   class streambuf
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

SerialBuf::~SerialBuf()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{

}

/****************************************************************************
*
* FILENAME
*   SSTREAM.H
*
* FUNCTION NAME
*   SerialBuf::sync
*
* PURPOSE
*   Synchronize buffer with I/O device.
*
* DESCRIPTION
*   Since there is no internal buffer, the stream is always in sync.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialBuf
*   class streambuf
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline virtual

int sync()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  return 0;
}

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialBuf::underflow
*
* PURPOSE
*   Called whenever the internal buffer underflows.
*
* DESCRIPTION
*   Since there is no internal buffer will, it will underflow on every input
*   operation. A byte is removed from the serial rece1ver queue, and
*   returned as if it had come from an internal buffer.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialBuf
*   class streambuf
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

int                     // RETURN: EOF if operation failed (never!)

SerialBuf::underflow();

/****************************************************************************
*
* CLASS END
*
*****************************************************************************/

};

/****************************************************************************
*
* FILENAME
*   SSTREAM.H
*
* CLASSNAME
*   SerialStream
*
* PURPOSE
*    A stream oriented serial class.
*
* DESCRIPTION
*    This class has all the features of class Serial and class iostream.
*
* INHERITANCE
*   public:
*   class Serial
*   class iostream
*
* MEMBER FUNCTIONS
*   public:
*   SerialBuf * SerialStream::rdbuf          ()
*               SerialStream::SerialStream   (SerialStream &)
*               SerialStream::SerialStream   (SerialStream::ComPort)
*               SerialStream::SerialStream   (SerialStream::ComPort,
*                                             unsigned, unsigned)
*               SerialStream::~SerialStream  ()
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* CLASS DEFINITION
*
*****************************************************************************/

class SerialStream
: public Serial,
  public iostream
{
  private:
  SerialBuf *serialBuf;  // pointer to class SerialBuf instance

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialStream::readbuf
*
* PURPOSE
*   Returns a pointer to the internal SerialBuf.
*
* DESCRIPTION
*   This function is included for compatibility, and the elimination of the
*   need to cast from a streambuf to a SerialBuf.
*
* EXTERNAL REFERENCES
*   class SerialBuf
*   class SerialStream
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

SerialBuf *     // RETURN: Pointer to the inernal class SerialBuf instance

SerialStream::rdbuf()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return a pointer to the internal class SerialBuf instance
  return serialBuf;
}

/****************************************************************************
*
* FILENAME
*   SSTREAM.H
*
* FUNCTION NAME
*   SerialStream::SerialStream
*
* PURPOSE
*   Copy constructor for the class.
*
* DESCRIPTION
*   The copy constructor is declared but not defined in order to prevent
*   copying.
*
* EXTERNAL REFERENCES
*   class SerialBuf
*   class SerialStream
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

SerialStream::SerialStream
(
  SerialStream &            // INPUT: Instance to copy
);

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialStream::SerialStream
*
* PURPOSE
*   Constructor that does not enable port.
*
* DESCRIPTION
*   The port must be enabled before it can be used.
*
* EXTERNAL REFERENCES
*   class SerialBuf
*   class SerialStream
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

SerialStream::SerialStream
(
  SerialStream::ComPort comPort  // INPUT: COM port to be used
);

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialStream::SerialStream
*
* PURPOSE
*   Constructor that enables port.
*
* DESCRIPTION
*   The port is ready for use after it is constructed.
*
* EXTERNAL REFERENCES
*   class SerialBuf
*   class SerialStream
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

SerialStream
(
  ComPort comPort,   // INPUT: COM port to use
  unsigned rxQSize,  // INPUT: Size of receive queue
  unsigned txQSize   // INPUT: Size of transmit queue
);

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialStream::~SerialStream
*
* PURPOSE
*   Destructor for the class.
*
* DESCRIPTION
*   The internal SerialBuf is deallocated.
*
* EXTERNAL REFERENCES
*   class SerialBuf
*   class SerialStream
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: virtual

SerialStream::~SerialStream();


/****************************************************************************
*
* CLASS END
*
*****************************************************************************/

};

/****************************************************************************
*
* PREPROCESSOR EXCLUSION END
*
*****************************************************************************/

#endif

/****************************************************************************/
